import { boolean, date, number, object, string, ValidationError } from 'yup';
import { Request, Response } from 'express';
import prisma from '../client';

/**
 * Show all non-adopted animals.
 * Supports filter for isRequested, animal type and limit on maximum number of returned animals.
 */
export const list = async (req: Request, res: Response) => {
  const isRequested = req.query.isRequested || false;
  const limit = +(req.query.limit || 10);

  const type = req.query.type + '';
  const types = [ 'dog', 'cat' ];

  const animals = await prisma.animal.findMany({
    where: {
      acceptedRequestId: {
        not: isRequested ? null : undefined
      },
      adoptedAt: null,
      type: types.includes(type) ? type : undefined
    },
    take: limit
  });

  return res.send({
    status: 'success',
    data: animals
  });
};

/**
 * Animal schema.
 */
const animalSchema = object({
  name: string().required(),
  description: string().min(10).default('Popis bude pridaný'),
  age: number().positive().required(),
  picture: string().default('/images/animal.webp'),
  type: string().matches(/(cat|dog)/).default('dog'),
  sex: boolean().required(),
  addedAt: date().required(),
});


/**
 * Store new animal.
 */
export const store = async (req: Request, res: Response) => {
  try {
    const data = await animalSchema.validate(req.body);
    const animal = await prisma.animal.create({
      data
    });

    return res.status(201).send({
      status: 'success',
      data: animal,
      message: 'Animal stored in system'
    });
  } catch (e) {
    if (e instanceof ValidationError) {
      return res.status(400).send({
        status: 'error',
        data: e.errors,
        message: e.message
      });
    }

    return res.status(500).send({
      status: 'error',
      data: {},
      message: 'Something went wrong'
    });
  }
};

/**
 * Find animal by id.
 * If no animal is found, return status code 404.
 */
export const get = async (req: Request, res: Response) => {
  const id = req.params.id;

  const animal = await prisma.animal.findUnique({
    where: {
      id: id
    }
  });

  if (!animal) {
    return res.status(404).send({
      status: 'error',
      data: {},
      message: 'Animal was not found'
    });
  }

  return res.send({
    status: 'success',
    data: animal
  });
};
