import { Request, Response } from 'express';
import { object, string, boolean, ValidationError } from 'yup';
import prisma from '../client';

/**
 * Request schema
 */
const requestSchema = object({
  from: string().required(),
  isAccepted: boolean().default(false),
  animalId: string().required().uuid(),
});

/**
 * Set adoption request as accepted
 */
export const partialUpdate = async (req: Request, res: Response) => {
  const request = await prisma.request.findUnique({
    where: {
      id: req.params.id
    }
  });

  await prisma.animal.update({
    data: {
      acceptedRequest: {
        connect: {
          id: req.params.id
        }
      },
      adoptedAt: new Date()
    },
    where: {
      id: request?.animalId
    }
  });

  const updatedRequest = await prisma.request.update({
    data: {
      isAccepted: true
    },
    where: {
      id: req.params.id
    }
  })

  return res.send({
    status: "success",
    data: updatedRequest,
    message: "Request updated"
  })
}

/**
 * Update all fields of request
 */
export const update = async (req: Request, res: Response) => {
  try {
    const requestData = await requestSchema.validate(req.body);
    const request = await prisma.request.update({
      data: requestData,
      where: {
        id: req.params.id
      }
    })

    return res.status(200).send({
      status: "success",
      data: request,
      message: "Request updated"
    })
  } catch (e) {
    if (e instanceof ValidationError) {
      return res.status(400).send({
        status: "error",
        data: e.errors,
        message: e.message
      });
    }

    return res.status(500).send({
      status: "error",
      data: {},
      message: "Something went wrong"
    });
  }
}

/**
 * Store new request
 * Requirement: Request adoption
 */
export const store = async (req: Request, res: Response) => {
  try {
    const data = await requestSchema.validate(req.body);
    const request = await prisma.request.create({
      data
    });

    return res.status(201).send({
      status: "success",
      data: request,
      message: "Request stored in system"
    })
  } catch (e) {
    if (e instanceof ValidationError) {
      return res.status(400).send({
        status: "error",
        data: e.errors,
        message: e.message
      });
    }

    return res.status(500).send({
      status: "error",
      data: {},
      message: "Something went wrong"
    });
  }
}

/**
 * Show all requests
 */
export const list = async (req: Request, res: Response) => {
  const requests = await prisma.request.findMany({
    where: {
      deletedAt: null
    }
  });

  return res.send({
    status: "success",
    data: requests
  })
}

/**
 * Show single request
 */
export const show = async (req: Request, res: Response) => {
  const request = await prisma.request.findUnique({
    where: {
      id: req.params.id
    }
  });

  return res.send({
    status: "success",
    data: request
  })
}

/**
 * Delete a request
 */
export const destroy = async (req: Request, res: Response) => {
  const request = await prisma.request.update({
    data: {
      deletedAt: new Date()
    },
    where: {
      id: req.params.id
    }
  });

  return res.send({
    status: "success",
    data: request,
    message: "Request removed"
  })
}
